@@ -85,6 +85,23 @@ class WidgetContainerMixin(object):
                 w.focus_position = p # modifies w.focus
             w = w.focus.base_widget
 
+    def get_focus_widgets(self):
+        """
+        Return the .focus values starting from this container
+        and proceeding along each child widget until reaching a leaf
+        (non-container) widget.
+
+        Note that the list does not contain the topmost container widget
+        (i.e, on which this method is called), but does include the
+        lowest leaf widget.
+        """
+        out = []
+        w = self
+        while True:
+            w = w.base_widget.focus
+            if w is None:
+                return out
+            out.append(w)
 
 class WidgetContainerListContentsMixin(object):
     """
@@ -343,6 +360,10 @@ class GridFlow(WidgetWrap, WidgetContainerMixin, WidgetContainerListContentsMixi
                 pad.original_widget=w
             pad.width = used_space - self.h_sep
 
+        if self.v_sep:
+            # remove first divider
+            del p.contents[:1]
+
         return p
 
     def _set_focus_from_display_widget(self):
@@ -1972,7 +1993,8 @@ class Columns(Widget, WidgetContainerMixin, WidgetContainerListContentsMixin):
         maxcol = size[0]
         # FIXME: get rid of this check and recalculate only when
         # a 'pack' widget has been modified.
-        if maxcol == self._cache_maxcol and not PACK in self.column_types:
+        if maxcol == self._cache_maxcol and not any(
+                t == PACK for w, (t, n, b) in self.contents):
             return self._cache_column_widths
 
         widths = []
