@@ -30,6 +30,7 @@ from urwid.container import Pile, Columns
 from urwid.display_common import AttrSpec
 from urwid.decoration import WidgetDecoration
 
+
 class BigText(Widget):
     _sizing = frozenset([FIXED])
 
@@ -40,42 +41,42 @@ class BigText(Widget):
         """
         self.set_font(font)
         self.set_text(markup)
-    
+
     def set_text(self, markup):
         self.text, self.attrib = decompose_tagmarkup(markup)
         self._invalidate()
-    
+
     def get_text(self):
         """
         Returns (text, attributes).
         """
         return self.text, self.attrib
-    
+
     def set_font(self, font):
         self.font = font
         self._invalidate()
-    
+
     def pack(self, size=None, focus=False):
         rows = self.font.height
         cols = 0
         for c in self.text:
             cols += self.font.char_width(c)
         return cols, rows
-    
+
     def render(self, size, focus=False):
-        fixed_size(size) # complain if parameter is wrong
+        fixed_size(size)  # complain if parameter is wrong
         a = None
         ai = ak = 0
         o = []
         rows = self.font.height
-        attrib = self.attrib+[(None,len(self.text))]
+        attrib = self.attrib + [(None, len(self.text))]
         for ch in self.text:
             if not ak:
                 a, ak = attrib[ai]
                 ai += 1
             ak -= 1
             width = self.font.char_width(ch)
-            if not width: 
+            if not width:
                 # ignore invalid characters
                 continue
             c = self.font.render(ch)
@@ -86,8 +87,8 @@ class BigText(Widget):
         if o:
             canv = CanvasJoin(o)
         else:
-            canv = TextCanvas([""]*rows, maxcol=0, 
-                check_width=False)
+            canv = TextCanvas([""] * rows, maxcol=0,
+                              check_width=False)
             canv = CompositeCanvas(canv)
         canv.set_depends([])
         return canv
@@ -139,7 +140,7 @@ class LineBox(WidgetDecoration, WidgetWrap):
             ('fixed', 1, lline),
             original_widget,
             ('fixed', 1, rline),
-        ], box_columns = [0,2], focus_column=1)
+        ], box_columns=[0, 2], focus_column=1)
 
         bottom = Columns([
             ('fixed', 1, blcorner), bline, ('fixed', 1, brcorner)
@@ -160,6 +161,7 @@ class LineBox(WidgetDecoration, WidgetWrap):
         self.title_widget.set_text(self.format_title(text))
         self.tline_widget._invalidate()
 
+
 class BarGraphMeta(WidgetMeta):
     """
     Detect subclass get_data() method and dynamic change to
@@ -178,6 +180,7 @@ class BarGraphMeta(WidgetMeta):
             lambda self: self._get_data,
             nocache_bargraph_get_data)
 
+
 def nocache_bargraph_get_data(self, get_data_fn):
     """
     Disable caching on this bargraph because get_data_fn needs
@@ -197,7 +200,7 @@ class BarGraph(Widget):
     ignore_focus = True
 
     eighths = u' ▁▂▃▄▅▆▇'
-    hlines  = u'_⎺⎻─⎼⎽'
+    hlines = u'_⎺⎻─⎼⎽'
 
     def __init__(self, attlist, hatt=None, satt=None):
         """
@@ -205,11 +208,11 @@ class BarGraph(Widget):
         see set_segment_attributes for a description of the parameters.
         """
 
-        self.set_segment_attributes( attlist, hatt, satt )
+        self.set_segment_attributes(attlist, hatt, satt)
         self.set_data([], 1, None)
         self.set_bar_width(None)
 
-    def set_segment_attributes(self, attlist, hatt=None, satt=None ):
+    def set_segment_attributes(self, attlist, hatt=None, satt=None):
         """
         :param attlist: list containing display attribute or
                         (display attribute, character) tuple for background,
@@ -247,10 +250,10 @@ class BarGraph(Widget):
         self.attr = []
         self.char = []
         if len(attlist) < 2:
-            raise BarGraphError, "attlist must include at least background and seg1: %r" % (attlist,)
+            raise BarGraphError("attlist must include at least background and seg1: %r" % (attlist,))
         assert len(attlist) >= 2, 'must at least specify bg and fg!'
         for a in attlist:
-            if type(a)!=tuple:
+            if type(a) != tuple:
                 self.attr.append(a)
                 self.char.append(' ')
             else:
@@ -261,39 +264,36 @@ class BarGraph(Widget):
         self.hatt = []
         if hatt is None:
             hatt = [self.attr[0]]
-        elif type(hatt)!=list:
+        elif type(hatt) != list:
             hatt = [hatt]
         self.hatt = hatt
-        
+
         if satt is None:
             satt = {}
         for i in satt.items():
             try:
-                (fg,bg), attr = i
+                (fg, bg), attr = i
             except ValueError:
-                raise BarGraphError, "satt not in (fg,bg:attr) form: %r"%(i,)
+                raise BarGraphError("satt not in (fg,bg:attr) form: %r" % (i,))
             if type(fg) != int or fg >= len(attlist):
-                raise BarGraphError, "fg not valid integer: %r"%(fg,)
+                raise BarGraphError("fg not valid integer: %r" % (fg,))
             if type(bg) != int or bg >= len(attlist):
-                raise BarGraphError, "bg not valid integer: %r"%(fg,)
-            if fg<=bg:
-                raise BarGraphError, "fg (%s) not > bg (%s)" %(fg,bg)
+                raise BarGraphError("bg not valid integer: %r" % (fg,))
+            if fg <= bg:
+                raise BarGraphError("fg (%s) not > bg (%s)" % (fg, bg))
         self.satt = satt
-            
-            
-        
-    
+
     def set_data(self, bardata, top, hlines=None):
         """
         Store bar data, bargraph top and horizontal line positions.
-        
+
         bardata -- a list of bar values.
         top -- maximum value for segments within bardata
         hlines -- None or a bar value marking horizontal line positions
 
-        bar values are [ segment1, segment2, ... ] lists where top is 
+        bar values are [ segment1, segment2, ... ] lists where top is
         the maximal value corresponding to the top of the bar graph and
-        segment1, segment2, ... are the values for the top of each 
+        segment1, segment2, ... are the values for the top of each
         segment of this bar.  Simple bar graphs will only have one
         segment in each bar value.
 
@@ -302,30 +302,30 @@ class BarGraph(Widget):
         and it will have a second segment that starts at 30%.
         """
         if hlines is not None:
-            hlines = hlines[:] # shallow copy
+            hlines = hlines[:]  # shallow copy
             hlines.sort()
         self.data = bardata, top, hlines
         self._invalidate()
-    
+
     def _get_data(self, size):
         """
         Return (bardata, top, hlines)
-        
+
         This function is called by render to retrieve the data for
         the graph. It may be overloaded to create a dynamic bar graph.
-        
-        This implementation will truncate the bardata list returned 
+
+        This implementation will truncate the bardata list returned
         if not all bars will fit within maxcol.
         """
         (maxcol, maxrow) = size
         bardata, top, hlines = self.data
-        widths = self.calculate_bar_widths((maxcol,maxrow),bardata)
-        
+        widths = self.calculate_bar_widths((maxcol, maxrow), bardata)
+
         if len(bardata) > len(widths):
             return bardata[:len(widths)], top, hlines
 
         return bardata, top, hlines
-    
+
     def set_bar_width(self, width):
         """
         Set a preferred bar width for calculate_bar_widths to use.
@@ -335,23 +335,23 @@ class BarGraph(Widget):
         assert width is None or width > 0
         self.bar_width = width
         self._invalidate()
-    
+
     def calculate_bar_widths(self, size, bardata):
         """
         Return a list of bar widths, one for each bar in data.
-        
-        If self.bar_width is None this implementation will stretch 
+
+        If self.bar_width is None this implementation will stretch
         the bars across the available space specified by maxcol.
         """
         (maxcol, maxrow) = size
-        
+
         if self.bar_width is not None:
             return [self.bar_width] * min(
-                len(bardata), maxcol/self.bar_width )
-        
+                len(bardata), maxcol / self.bar_width)
+
         if len(bardata) >= maxcol:
             return [1] * maxcol
-        
+
         widths = []
         grow = maxcol
         remain = len(bardata)
@@ -361,40 +361,39 @@ class BarGraph(Widget):
             grow -= w
             remain -= 1
         return widths
-        
 
     def selectable(self):
         """
         Return False.
         """
         return False
-    
+
     def use_smoothed(self):
-        return self.satt and get_encoding_mode()=="utf8"
-        
+        return self.satt and get_encoding_mode() == "utf8"
+
     def calculate_display(self, size):
         """
         Calculate display data.
         """
         (maxcol, maxrow) = size
-        bardata, top, hlines = self.get_data( (maxcol, maxrow) )
-        widths = self.calculate_bar_widths( (maxcol, maxrow), bardata )
+        bardata, top, hlines = self.get_data((maxcol, maxrow))
+        widths = self.calculate_bar_widths((maxcol, maxrow), bardata)
 
         if self.use_smoothed():
             disp = calculate_bargraph_display(bardata, top, widths,
-                maxrow * 8 )
-            disp = self.smooth_display( disp )
-    
+                                              maxrow * 8)
+            disp = self.smooth_display(disp)
+
         else:
             disp = calculate_bargraph_display(bardata, top, widths,
-                maxrow )
+                                              maxrow)
 
         if hlines:
-            disp = self.hlines_display( disp, top, hlines, maxrow )
-        
+            disp = self.hlines_display(disp, top, hlines, maxrow)
+
         return disp
 
-    def hlines_display(self, disp, top, hlines, maxrow ):
+    def hlines_display(self, disp, top, hlines, maxrow):
         """
         Add hlines to display structure represented as bar_type tuple
         values:
@@ -405,23 +404,23 @@ class BarGraph(Widget):
         """
         if self.use_smoothed():
             shiftr = 0
-            r = [    (0.2, 1),
-                (0.4, 2),
-                (0.6, 3),
-                (0.8, 4),
-                (1.0, 5),]
+            r = [(0.2, 1),
+                 (0.4, 2),
+                 (0.6, 3),
+                 (0.8, 4),
+                 (1.0, 5), ]
         else:
             shiftr = 0.5
-            r = [    (1.0, 0), ]
+            r = [(1.0, 0), ]
 
         # reverse the hlines to match screen ordering
         rhl = []
         for h in hlines:
-            rh = float(top-h) * maxrow / top - shiftr
+            rh = float(top - h) * maxrow / top - shiftr
             if rh < 0:
                 continue
             rhl.append(rh)
-    
+
         # build a list of rows that will have hlines
         hrows = []
         last_i = -1
@@ -429,47 +428,46 @@ class BarGraph(Widget):
             i = int(rh)
             if i == last_i:
                 continue
-            f = rh-i
+            f = rh - i
             for spl, chnum in r:
                 if f < spl:
-                    hrows.append( (i, chnum) )
+                    hrows.append((i, chnum))
                     break
             last_i = i
 
         # fill hlines into disp data
-        def fill_row( row, chnum ):
+        def fill_row(row, chnum):
             rout = []
             for bar_type, width in row:
-                if (type(bar_type) == int and 
-                        len(self.hatt) > bar_type ):
-                    rout.append( ((bar_type, chnum), width))
+                if (type(bar_type) == int and
+                        len(self.hatt) > bar_type):
+                    rout.append(((bar_type, chnum), width))
                     continue
-                rout.append( (bar_type, width))
+                rout.append((bar_type, width))
             return rout
-            
+
         o = []
         k = 0
         rnum = 0
         for y_count, row in disp:
             if k >= len(hrows):
-                o.append( (y_count, row) )
+                o.append((y_count, row))
                 continue
             end_block = rnum + y_count
             while k < len(hrows) and hrows[k][0] < end_block:
                 i, chnum = hrows[k]
-                if i-rnum > 0:
-                    o.append( (i-rnum, row) )
-                o.append( (1, fill_row( row, chnum ) ) )
-                rnum = i+1
+                if i - rnum > 0:
+                    o.append((i - rnum, row))
+                o.append((1, fill_row(row, chnum)))
+                rnum = i + 1
                 k += 1
             if rnum < end_block:
-                o.append( (end_block-rnum, row) )
+                o.append((end_block - rnum, row))
                 rnum = end_block
-        
+
         #assert 0, o
         return o
 
-        
     def smooth_display(self, disp):
         """
         smooth (col, row*8) display into (col, row) display using
@@ -480,50 +478,50 @@ class BarGraph(Widget):
         1-7 is the vertical eighth character to use.
         """
         o = []
-        r = 0 # row remainder
-        def seg_combine( (bt1,w1), (bt2,w2) ):
-            if (bt1,w1) == (bt2,w2):
-                return (bt1,w1), None, None
-            wmin = min(w1,w2)
+        r = 0  # row remainder
+
+        def seg_combine((bt1, w1), (bt2, w2)):
+            if (bt1, w1) == (bt2, w2):
+                return (bt1, w1), None, None
+            wmin = min(w1, w2)
             l1 = l2 = None
-            if w1>w2:
-                l1 = (bt1, w1-w2)
-            elif w2>w1:
-                l2 = (bt2, w2-w1)
-            if type(bt1)==tuple:
-                return (bt1,wmin), l1, l2
-            if not self.satt.has_key( (bt2, bt1) ):
-                if r<4:
-                    return (bt2,wmin), l1, l2
-                return (bt1,wmin), l1, l2
-            return ((bt2, bt1, 8-r), wmin), l1, l2
-                    
-        def row_combine_last( count, row ):
+            if w1 > w2:
+                l1 = (bt1, w1 - w2)
+            elif w2 > w1:
+                l2 = (bt2, w2 - w1)
+            if type(bt1) == tuple:
+                return (bt1, wmin), l1, l2
+            if (bt2, bt1) not in self.satt:
+                if r < 4:
+                    return (bt2, wmin), l1, l2
+                return (bt1, wmin), l1, l2
+            return ((bt2, bt1, 8 - r), wmin), l1, l2
+
+        def row_combine_last(count, row):
             o_count, o_row = o[-1]
-            row = row[:] # shallow copy, so we don't destroy orig.
+            row = row[:]  # shallow copy, so we don't destroy orig.
             o_row = o_row[:]
             l = []
             while row:
                 (bt, w), l1, l2 = seg_combine(
-                    o_row.pop(0), row.pop(0) )
+                    o_row.pop(0), row.pop(0))
                 if l and l[-1][0] == bt:
-                    l[-1] = (bt, l[-1][1]+w)
+                    l[-1] = (bt, l[-1][1] + w)
                 else:
                     l.append((bt, w))
                 if l1:
-                    o_row = [l1]+o_row
+                    o_row = [l1] + o_row
                 if l2:
-                    row = [l2]+row
-            
+                    row = [l2] + row
+
             assert not o_row
-            o[-1] = ( o_count + count, l )
-            
-        
+            o[-1] = (o_count + count, l)
+
         # regroup into actual rows (8 disp rows == 1 actual row)
         for y_count, row in disp:
             if r:
-                count = min( 8-r, y_count )
-                row_combine_last( count, row )
+                count = min(8 - r, y_count)
+                row_combine_last(count, row)
                 y_count -= count
                 r += count
                 r = r % 8
@@ -532,22 +530,21 @@ class BarGraph(Widget):
             assert r == 0
             # copy whole blocks
             if y_count > 7:
-                o.append( (y_count // 8 * 8 , row) )
-                y_count = y_count %8
+                o.append((y_count // 8 * 8, row))
+                y_count = y_count % 8
                 if not y_count:
                     continue
-            o.append( (y_count, row) )
+            o.append((y_count, row))
             r = y_count
-        return [(y // 8, row) for (y,row) in o]
-            
-            
+        return [(y // 8, row) for (y, row) in o]
+
     def render(self, size, focus=False):
         """
         Render BarGraph.
         """
         (maxcol, maxrow) = size
-        disp = self.calculate_display( (maxcol,maxrow) )
-        
+        disp = self.calculate_display((maxcol, maxrow))
+
         combinelist = []
         for y_count, row in disp:
             l = []
@@ -555,37 +552,36 @@ class BarGraph(Widget):
                 if type(bar_type) == tuple:
                     if len(bar_type) == 3:
                         # vertical eighths
-                        fg,bg,k = bar_type
-                        a = self.satt[(fg,bg)]
+                        fg, bg, k = bar_type
+                        a = self.satt[(fg, bg)]
                         t = self.eighths[k] * width
                     else:
                         # horizontal lines
-                        bg,k = bar_type
+                        bg, k = bar_type
                         a = self.hatt[bg]
                         t = self.hlines[k] * width
                 else:
                     a = self.attr[bar_type]
                     t = self.char[bar_type] * width
-                l.append( (a, t) )
-            c = Text(l).render( (maxcol,) )
+                l.append((a, t))
+            c = Text(l).render((maxcol,))
             assert c.rows() == 1, "Invalid characters in BarGraph!"
             combinelist += [(c, None, False)] * y_count
-            
+
         canv = CanvasCombine(combinelist)
         return canv
 
 
-
-def calculate_bargraph_display( bardata, top, bar_widths, maxrow ):
+def calculate_bargraph_display(bardata, top, bar_widths, maxrow):
     """
     Calculate a rendering of the bar graph described by data, bar_widths
     and height.
-    
+
     bardata -- bar information with same structure as BarGraph.data
     top -- maximal value for bardata segments
     bar_widths -- list of integer column widths for each bar
     maxrow -- rows for display of bargraph
-    
+
     Returns a structure as follows:
       [ ( y_count, [ ( bar_type, width), ... ] ), ... ]
 
@@ -606,24 +602,25 @@ def calculate_bargraph_display( bardata, top, bar_widths, maxrow ):
     maxcol = sum(bar_widths)
 
     # build intermediate data structure
-    rows = [None]*maxrow
-    def add_segment( seg_num, col, row, width, rows=rows ):
+    rows = [None] * maxrow
+
+    def add_segment(seg_num, col, row, width, rows=rows):
         if rows[row]:
             last_seg, last_col, last_end = rows[row][-1]
             if last_end > col:
                 if last_col >= col:
                     del rows[row][-1]
                 else:
-                    rows[row][-1] = ( last_seg, 
-                        last_col, col)
+                    rows[row][-1] = (last_seg,
+                                     last_col, col)
             elif last_seg == seg_num and last_end == col:
-                rows[row][-1] = ( last_seg, last_col, 
-                    last_end+width)
+                rows[row][-1] = (last_seg, last_col,
+                                 last_end + width)
                 return
         elif rows[row] is None:
             rows[row] = []
-        rows[row].append( (seg_num, col, col+width) )
-    
+        rows[row].append((seg_num, col, col + width))
+
     col = 0
     barnum = 0
     for bar in bardata:
@@ -632,27 +629,27 @@ def calculate_bargraph_display( bardata, top, bar_widths, maxrow ):
             continue
         # loop through in reverse order
         tallest = maxrow
-        segments = scale_bar_values( bar, top, maxrow )
-        for k in range(len(bar)-1,-1,-1): 
+        segments = scale_bar_values(bar, top, maxrow)
+        for k in range(len(bar) - 1, -1, -1):
             s = segments[k]
-            
-            if s >= maxrow: 
+
+            if s >= maxrow:
                 continue
             if s < 0:
                 s = 0
             if s < tallest:
                 # add only properly-overlapped bars
                 tallest = s
-                add_segment( k+1, col, s, width )
+                add_segment(k + 1, col, s, width)
         col += width
         barnum += 1
-    
+
     #print repr(rows)
     # build rowsets data structure
     rowsets = []
     y_count = 0
-    last = [(0,maxcol)]
-    
+    last = [(0, maxcol)]
+
     for r in rows:
         if r is None:
             y_count = y_count + 1
@@ -660,58 +657,58 @@ def calculate_bargraph_display( bardata, top, bar_widths, maxrow ):
         if y_count:
             rowsets.append((y_count, last))
             y_count = 0
-        
-        i = 0 # index into "last"
-        la, ln = last[i] # last attribute, last run length
-        c = 0 # current column
-        o = [] # output list to be added to rowsets
+
+        i = 0  # index into "last"
+        la, ln = last[i]  # last attribute, last run length
+        c = 0  # current column
+        o = []  # output list to be added to rowsets
         for seg_num, start, end in r:
-            while start > c+ln:
-                o.append( (la, ln) )
+            while start > c + ln:
+                o.append((la, ln))
                 i += 1
                 c += ln
                 la, ln = last[i]
-                
+
             if la == seg_num:
                 # same attribute, can combine
-                o.append( (la, end-c) )
+                o.append((la, end - c))
             else:
-                if start-c > 0:
-                    o.append( (la, start-c) )
-                o.append( (seg_num, end-start) )
-            
+                if start - c > 0:
+                    o.append((la, start - c))
+                o.append((seg_num, end - start))
+
             if end == maxcol:
                 i = len(last)
                 break
-            
+
             # skip past old segments covered by new one
-            while end >= c+ln:
+            while end >= c + ln:
                 i += 1
                 c += ln
                 la, ln = last[i]
 
             if la != seg_num:
-                ln = c+ln-end
+                ln = c + ln - end
                 c = end
                 continue
-            
+
             # same attribute, can extend
             oa, on = o[-1]
-            on += c+ln-end
+            on += c + ln - end
             o[-1] = oa, on
 
             i += 1
             c += ln
             if c == maxcol:
                 break
-            assert i<len(last), repr((on, maxcol))
+            assert i < len(last), repr((on, maxcol))
             la, ln = last[i]
-    
-        if i < len(last): 
-            o += [(la,ln)]+last[i+1:]    
+
+        if i < len(last):
+            o += [(la, ln)] + last[i + 1:]
         last = o
         y_count += 1
-    
+
     if y_count:
         rowsets.append((y_count, last))
 
@@ -731,8 +728,8 @@ class GraphVScale(Widget):
         This widget is a vertical scale for the BarGraph widget that
         can correspond to the BarGraph's horizontal lines
         """
-        self.set_scale( labels, top )
-    
+        self.set_scale(labels, top)
+
     def set_scale(self, labels, top):
         """
         set_scale( [(label1 position, label1 markup),...], top )
@@ -740,39 +737,41 @@ class GraphVScale(Widget):
         label markup -- text markup for this label
         top -- top y position
         """
-        
-        labels = labels[:] # shallow copy
+
+        labels = labels[:]  # shallow copy
         labels.sort()
         labels.reverse()
         self.pos = []
         self.txt = []
         for y, markup in labels:
             self.pos.append(y)
-            self.txt.append( Text(markup) )
+            self.txt.append(Text(markup))
         self.top = top
-        
+
     def selectable(self):
         """
         Return False.
         """
         return False
-    
+
     def render(self, size, focus=False):
         """
         Render GraphVScale.
         """
         (maxcol, maxrow) = size
-        pl = scale_bar_values( self.pos, self.top, maxrow )
+        pl = scale_bar_values(self.pos, self.top, maxrow)
 
         combinelist = []
         rows = 0
         for p, t in zip(pl, self.txt):
             p -= 1
-            if p >= maxrow: break
-            if p < rows: continue
+            if p >= maxrow:
+                break
+            if p < rows:
+                continue
             c = t.render((maxcol,))
             if p > rows:
-                run = p-rows
+                run = p - rows
                 c = CompositeCanvas(c)
                 c.pad_trim_top_bottom(run, 0)
             rows += c.rows()
@@ -818,13 +817,13 @@ class ProgressBar(Widget):
         self._done = done
         self.satt = satt
 
-    def set_completion(self, current ):
+    def set_completion(self, current):
         """
         current -- current progress
         """
         self._current = current
         self._invalidate()
-    current = property(lambda self:self._current, set_completion)
+    current = property(lambda self: self._current, set_completion)
 
     def _set_done(self, done):
         """
@@ -832,7 +831,7 @@ class ProgressBar(Widget):
         """
         self._done = done
         self._invalidate()
-    done = property(lambda self:self._done, _set_done)
+    done = property(lambda self: self._done, _set_done)
 
     def rows(self, size, focus=False):
         return 1
@@ -849,35 +848,36 @@ class ProgressBar(Widget):
         Render the progress bar.
         """
         (maxcol,) = size
-        txt = Text(self.get_text(), self.text_align, CLIP )
+        txt = Text(self.get_text(), self.text_align, CLIP)
         c = txt.render((maxcol,))
 
-        cf = float( self.current ) * maxcol / self.done
-        ccol = int( cf )
+        cf = float(self.current) * maxcol / self.done
+        ccol = int(cf)
         cs = 0
         if self.satt is not None:
             cs = int((cf - ccol) * 8)
         if ccol < 0 or (ccol == 0 and cs == 0):
-            c._attr = [[(self.normal,maxcol)]]
+            c._attr = [[(self.normal, maxcol)]]
         elif ccol >= maxcol:
-            c._attr = [[(self.complete,maxcol)]]
+            c._attr = [[(self.complete, maxcol)]]
         elif cs and c._text[0][ccol] == " ":
             t = c._text[0]
             cenc = self.eighths[cs].encode("utf-8")
-            c._text[0] = t[:ccol]+cenc+t[ccol+1:]
+            c._text[0] = t[:ccol] + cenc + t[ccol + 1:]
             a = []
             if ccol > 0:
-                a.append( (self.complete, ccol) )
-            a.append((self.satt,len(cenc)))
-            if maxcol-ccol-1 > 0:
-                a.append( (self.normal, maxcol-ccol-1) )
+                a.append((self.complete, ccol))
+            a.append((self.satt, len(cenc)))
+            if maxcol - ccol - 1 > 0:
+                a.append((self.normal, maxcol - ccol - 1))
             c._attr = [a]
             c._cs = [[(None, len(c._text[0]))]]
         else:
-            c._attr = [[(self.complete,ccol),
-                (self.normal,maxcol-ccol)]]
+            c._attr = [[(self.complete, ccol),
+                        (self.normal, maxcol - ccol)]]
         return c
-    
+
+
 class PythonLogo(Widget):
     _sizing = frozenset([FIXED])
 
@@ -908,6 +908,3 @@ class PythonLogo(Widget):
         """
         fixed_size(size)
         return self._canvas
-
-
-
