@@ -614,7 +614,7 @@ class Screen:
             background = "light gray"
         self.palette[name] = (foreground, background, mono)
 
-    def set_mouse_tracking(self):
+    def set_mouse_tracking(self, enable=True):
         """Not yet implemented"""
         pass
 
